@@ -43,7 +43,9 @@ from tornado import ioloop
 
 import inmanta.ast
 from inmanta import compiler, config, const, module, plugins, protocol
-from inmanta.agent import cache, handler
+from inmanta.agent import cache
+from inmanta.agent import config as inmanta_config
+from inmanta.agent import handler
 from inmanta.agent import io as agent_io
 from inmanta.agent.handler import HandlerContext, ResourceHandler
 from inmanta.const import ResourceState
@@ -55,14 +57,23 @@ from inmanta.protocol import json_encode
 from inmanta.resources import Resource
 
 if typing.TYPE_CHECKING:
+    # Local type stub for mypy that works with both pytest < 7 and pytest >=7
+    # https://docs.pytest.org/en/7.1.x/_modules/_pytest/legacypath.html#TempdirFactory
+    import py
     from inmanta.agent.io.local import IOBase
 
+    class TempdirFactory:
+        def mktemp(self, path: str) -> py.path.local:
+            ...
+
+
 from .handler import DATA
 from .parameters import (
     inm_install_mode,
     inm_mod_in_place,
     inm_mod_repo,
     inm_no_load_plugins,
+    inm_no_strict_deps_check,
     inm_venv,
 )
 from .test_parameter import ParameterNotSetException, TestParameterRegistry
@@ -125,7 +136,7 @@ def inmanta_plugins(
 
 @pytest.fixture()
 def project(
-    project_shared: "Project", capsys: "CaptureFixture"
+    project_shared: "Project", capsys: "CaptureFixture", set_inmanta_state_dir: None
 ) -> typing.Iterator["Project"]:
     DATA.clear()
     project_shared.clean()
@@ -136,7 +147,9 @@ def project(
 
 @pytest.fixture()
 def project_no_plugins(
-    project_shared_no_plugins: "Project", capsys: "CaptureFixture"
+    project_shared_no_plugins: "Project",
+    capsys: "CaptureFixture",
+    set_inmanta_state_dir: None,
 ) -> typing.Iterator["Project"]:
     warnings.warn(
         DeprecationWarning(
@@ -211,13 +224,33 @@ def get_project_repos(repo_options: typing.Sequence[str]) -> typing.Sequence[obj
 
 
 @pytest.fixture(scope="session")
-def project_factory(request: pytest.FixtureRequest) -> typing.Callable[[], "Project"]:
+def project_dir() -> str:
+    """
+    Returns the directory that will be used by the project_factory fixture to store the Inmanta project.
+    """
+    project_dir_path = tempfile.mkdtemp()
+
+    yield project_dir_path
+
+    try:
+        shutil.rmtree(project_dir_path)
+    except PermissionError:
+        LOGGER.warning(
+            "Cannot cleanup test project %s. This can be caused because we try to remove a virtual environment, "
+            "loaded by this python process. Try to use a shared environment with --venv",
+            project_dir_path,
+        )
+
+
+@pytest.fixture(scope="session")
+def project_factory(
+    request: pytest.FixtureRequest, project_dir: str
+) -> typing.Callable[[], "Project"]:
     """
     A factory that constructs a single Project.
     """
     _sys_path = sys.path
-    test_project_dir = tempfile.mkdtemp()
-    os.mkdir(os.path.join(test_project_dir, "libs"))
+    os.mkdir(os.path.join(project_dir, "libs"))
 
     repo_options = inm_mod_repo.resolve(request.config)
     repos: typing.Sequence[object] = get_project_repos(
@@ -240,7 +273,7 @@ def project_factory(request: pytest.FixtureRequest) -> typing.Callable[[], "Proj
         env_override: Optional[str] = str(inm_venv.resolve(request.config))
     except ParameterNotSetException:
         env_override = None
-    env_dir = os.path.join(test_project_dir, ".env")
+    env_dir = os.path.join(project_dir, ".env")
     if env_override and not os.path.isdir(env_override):
         raise Exception(f"Specified venv {env_override} does not exist")
     if env_override is not None:
@@ -250,11 +283,11 @@ def project_factory(request: pytest.FixtureRequest) -> typing.Callable[[], "Proj
             LOGGER.exception(
                 "Unable to use shared env (symlink creation from %s to %s failed).",
                 env_override,
-                os.path.join(test_project_dir, ".env"),
+                os.path.join(project_dir, ".env"),
             )
             raise
 
-    with open(os.path.join(test_project_dir, "project.yml"), "w+") as fd:
+    with open(os.path.join(project_dir, "project.yml"), "w+") as fd:
         metadata: typing.Mapping[str, object] = {
             "name": "testcase",
             "description": "Project for testcase",
@@ -265,16 +298,18 @@ def project_factory(request: pytest.FixtureRequest) -> typing.Callable[[], "Proj
         }
         yaml.dump(metadata, fd)
 
-    ensure_current_module_install(os.path.join(test_project_dir, "libs"), in_place)
+    ensure_current_module_install(os.path.join(project_dir, "libs"), in_place)
 
     def create_project(**kwargs: object):
         load_plugins = not inm_no_load_plugins.resolve(request.config)
+        no_strict_deps_check = inm_no_strict_deps_check.resolve(request.config)
         extended_kwargs: typing.Dict[str, object] = {
+            "no_strict_deps_check": no_strict_deps_check,
             "load_plugins": load_plugins,
             "env_path": env_dir,
             **kwargs,
         }
-        test_project = Project(test_project_dir, **extended_kwargs)
+        test_project = Project(project_dir, **extended_kwargs)
 
         # create the unittest module
         test_project.create_module(
@@ -287,15 +322,6 @@ def project_factory(request: pytest.FixtureRequest) -> typing.Callable[[], "Proj
 
     yield create_project
 
-    try:
-        shutil.rmtree(test_project_dir)
-    except PermissionError:
-        LOGGER.warning(
-            "Cannot cleanup test project %s. This can be caused because we try to remove a virtual environment, "
-            "loaded by this python process. Try to use a shared environment with --venv",
-            test_project_dir,
-        )
-
     sys.path = _sys_path
 
 
@@ -415,6 +441,15 @@ class ProjectLoader:
     _registered_plugins: typing.Dict[str, typing.Type[plugins.Plugin]] = {}
     _dynamic_modules: typing.Set[str] = set()
 
+    @classmethod
+    def reset(cls) -> None:
+        """
+        Fully resets the ProjectLoader. For normal pytest-inmanta use this is not required (or even desired). It is used for
+        resetting the singleton state in between distinct module tests for pytest-inmanta's own test suite.
+        """
+        cls._registered_plugins = {}
+        cls._dynamic_modules = set()
+
     @classmethod
     def load(cls, project: module.Project) -> None:
         """
@@ -593,6 +628,7 @@ class Project:
         project_dir: str,
         env_path: str,
         load_plugins: typing.Optional[bool] = True,
+        no_strict_deps_check: typing.Optional[bool] = False,
     ) -> None:
         """
         :param project_dir: Directory containing the Inmanta project.
@@ -601,6 +637,7 @@ class Project:
         """
         self._test_project_dir = project_dir
         self._env_path = env_path
+        self.no_strict_deps_check = no_strict_deps_check
         self._stdout: typing.Optional[str] = None
         self._stderr: typing.Optional[str] = None
         self.types: typing.Optional[typing.Dict[str, inmanta.ast.Type]] = None
@@ -651,7 +688,7 @@ class Project:
 
     def _create_project_and_load(self, model: str) -> module.Project:
         """
-        This method doesn the following:
+        This method does the following:
         * Add the given model file to the Inmanta project
         * Install the module dependencies
         * Load the project
@@ -666,12 +703,20 @@ class Project:
             module.Project.__init__
         )
         # The venv_path parameter only exists on ISO5+
+
         extra_kwargs_init = (
             {"venv_path": self._env_path}
             if "venv_path" in signature_init.parameters.keys()
             else {}
         )
-        test_project = module.Project(self._test_project_dir, **extra_kwargs_init)
+
+        if "strict_deps_check" in signature_init.parameters.keys():
+            extra_kwargs_init["strict_deps_check"] = not self.no_strict_deps_check
+
+        test_project = module.Project(
+            self._test_project_dir,
+            **extra_kwargs_init,
+        )
 
         ProjectLoader.load(test_project)
 
@@ -1130,3 +1175,19 @@ license: Test License
     def finalize_all_handlers(self) -> None:
         for handler_instance in self._handlers:
             self.finalize_handler(handler_instance)
+
+
+@pytest.fixture(scope="function")
+def inmanta_state_dir(tmpdir_factory: "TempdirFactory") -> Iterator[str]:
+    """
+    This fixture can be overridden in the conftest of any individual project
+    in order to set the Inmanta state directory at the desired level.
+    """
+    inmanta_state_dir = tmpdir_factory.mktemp("inmanta_state_dir")
+    yield str(inmanta_state_dir)
+    inmanta_state_dir.remove()
+
+
+@pytest.fixture
+def set_inmanta_state_dir(inmanta_state_dir: str) -> None:
+    inmanta_config.state_dir.set(inmanta_state_dir)
